@@ -1,8 +1,12 @@
+from abc import ABCMeta
 import typing as ty
 import attrs
 from .utils import describe_task
 from .exceptions import FileFormatsError
 
+if ty.TYPE_CHECKING:
+    from .datatype import DataType
+
 
 @attrs.define
 class ConverterWrapper:
@@ -11,18 +15,18 @@ class ConverterWrapper:
     """
 
     task_spec: ty.Callable
-    in_file: str = None
-    out_file: str = None
+    in_file: ty.Optional[str] = None
+    out_file: ty.Optional[str] = None
 
     def __call__(self, name=None, **kwargs):
         from pydra.engine import Workflow
 
         if name is None:
             name = f"{self.task_spec.__name__}_wrapper"
-        wf = Workflow(
-            name=name, input_spec=list(set(["in_file"] + list(kwargs))), **kwargs
-        )
-        wf.add(self.task_spec(name="task", **{self.in_file: wf.lzin.in_file}))
+        wf = Workflow(name=name, input_spec=["in_file"])
+        task_kwargs = {self.in_file: wf.lzin.in_file}
+        task_kwargs.update(kwargs)
+        wf.add(self.task_spec(name="task", **task_kwargs))
         wf.set_output([("out_file", getattr(wf.task.lzout, self.out_file))])
         return wf
 
@@ -41,45 +45,29 @@ class SubtypeVar:
 
     converters = {}
 
-    def __init__(self, name: str, base: type):
-        self.name = name
-        self.base = base
-
-    def __str__(self):
-        return f"{self.base.__name__}:{self.name}"
-
-    def __repr__(self):
-        return f"SubtypeVar({str(self)})"
-
-    def __hash__(self):
-        return hash((type(self), self.base, self.name))
-
-    @property
-    def __name__(self):
-        return self.name
-
-    def issubtype(self, super_type: type, allow_same: bool = True):
-        """Check to see whether datatype class is a subtype of a given super class.
-        In this case the subtype is expected to be able to be treated as if it was
-        the super class.
-
-        Overridden in the ``WithQualifiers`` mixin to add support for
-        qualified subtypes
+    @classmethod
+    def new(cls, name: str, klass: ty.Type["DataType"]) -> "SubtypeVar":
+        """Create a new subtype
 
         Parameters
         ----------
-        super_type : type
-            the class to check whether the given class is a subtype of
-        allow_same : bool, optional
-            whether there is a match if the classes are the same, by default True
+        name : str
+            name for the subtype
+        klass : ty.Type[DataType]
+            the class to sub-type
 
         Returns
         -------
-        is_subtype : bool
-            whether or not the current class can be considered a subtype of the super (or
-            is the super itself)
+        SubtypeVar
+            a sub-type that is
         """
-        return self.base.issubtype(super_type, allow_same=allow_same)
+        return ABCMeta(name, (cls, klass), {"bound": klass})
+
+    @classmethod
+    def __subclasshook__(cls, subclass: type) -> bool:
+        if issubclass(subclass, SubtypeVar):
+            return issubclass(subclass.bound, cls.bound)
+        return type.__subclasscheck__(cls, subclass)
 
     @classmethod
     def get_converter_tuples(
@@ -88,20 +76,20 @@ class SubtypeVar:
         # check to see whether there are converters from a base class of the source
         # format
         available_converters = []
-        if source_format.is_qualified:
+        if source_format.is_classified:
             for template_source_format, converter in cls.converters.items():
-                if not template_source_format.unqualified.issubtype(
-                    source_format.unqualified
+                if not issubclass(
+                    template_source_format.unclassified, source_format.unclassified
                 ):
                     continue
-                assert len(template_source_format.wildcard_qualifiers()) == 1
-                non_wildcards = template_source_format.non_wildcard_qualifiers()
-                if not non_wildcards.issubset(source_format.qualifiers):
+                assert len(template_source_format.wildcard_classifiers()) == 1
+                non_wildcards = template_source_format.non_wildcard_classifiers()
+                if not non_wildcards.issubset(source_format.classifiers):
                     continue
                 from_types = tuple(
-                    set(source_format.qualifiers).difference(non_wildcards)
+                    set(source_format.classifiers).difference(non_wildcards)
                 )
-                if any(q.issubtype(target_format) for q in from_types):
+                if any(issubclass(q, target_format) for q in from_types):
                     available_converters.append(converter)
         return available_converters
 
@@ -130,18 +118,18 @@ class SubtypeVar:
             if there is already a converter registered between the two types
         """
         # Ensure "converters" dict is defined in the target class and not in a superclass
-        if len(source_format.wildcard_qualifiers()) > 1:
+        if len(source_format.wildcard_classifiers()) > 1:
             raise FileFormatsError(
                 "Cannot register a conversion to a generic type from a type with more "
-                f"than one wildcard {source_format} ({list(source_format.wildcard_qualifiers())})"
+                f"than one wildcard {source_format} ({list(source_format.wildcard_classifiers())})"
             )
         prev_registered = [
             f
             for f in cls.converters
             if (
-                f.unqualified is source_format.unqualified
-                and f.non_wildcard_qualifiers()
-                == source_format.non_wildcard_qualifiers()
+                f.unclassified is source_format.unclassified
+                and f.non_wildcard_classifiers()
+                == source_format.non_wildcard_classifiers()
             )
         ]
         assert len(prev_registered) <= 1
@@ -150,7 +138,7 @@ class SubtypeVar:
             prev_task = cls.converters[prev][0]
             raise FileFormatsError(
                 f"There is already a converter registered from {prev} "
-                f"to the generic type '{tuple(prev.wildcard_qualifiers())[0]}':"
+                f"to the generic type '{tuple(prev.wildcard_classifiers())[0]}':"
                 f"{describe_task(prev_task)}"
             )
 
