@@ -1,8 +1,12 @@
 import importlib
+import typing as ty
+import functools
 import attrs
-from .base import DataType, REQUIRED_ANNOTATION, CHECK_ANNOTATION
+import urllib.error
+from .datatype import DataType
 from .converter import ConverterWrapper
-from .exceptions import FormatConversionError
+from .exceptions import FormatConversionError, FileFormatsExtrasError
+from .utils import import_extras_module, check_package_exists_on_pypi
 
 
 __all__ = ["required", "check", "converter"]
@@ -24,7 +28,7 @@ def required(prop):
         to pass to the operator (the value of the property will be the first operand)
     """
 
-    prop.fget.__annotations__[REQUIRED_ANNOTATION] = None
+    prop.fget.__annotations__[DataType.REQUIRED_ANNOTATION] = None
     return prop
 
 
@@ -37,7 +41,7 @@ def check(method):
     method : Function
         the method to mark as a check
     """
-    method.__annotations__[CHECK_ANNOTATION] = None
+    method.__annotations__[DataType.CHECK_ANNOTATION] = None
     return method
 
 
@@ -103,14 +107,16 @@ def converter(
                 source = eval(source, module_dict)
             if isinstance(target, str):
                 target = eval(target, module_dict)
-        if not target.issubtype(DataType):
+        if not issubclass(target, DataType):
             raise FormatConversionError(
                 f"Target file format '{target.__name__}' is not of subtype of "
                 "fileformats.core.DataType"
             )
         if in_file != "in_file" or out_file_local != "out_file":
             task_spec = ConverterWrapper(
-                task_spec, in_file=in_file, out_file=out_file_local
+                task_spec,
+                in_file=in_file,
+                out_file=out_file_local,
             )
         target.register_converter(
             source_format=source, converter_tuple=(task_spec, converter_kwargs)
@@ -118,3 +124,39 @@ def converter(
         return task_spec
 
     return decorator if task_spec is None else decorator(task_spec)
+
+
+def extra(method: ty.Callable):
+    """A decorator which uses singledispatch to facilitate the registering of
+    "extra" functionality in external packages (e.g. "fileformats-extras")"""
+
+    dispatch_method = functools.singledispatch(method)
+
+    @functools.wraps(method)
+    def decorated(obj, *args, **kwargs):
+        cls = type(obj)
+        extras = []
+        for tp in cls.referenced_types():
+            extras.append(import_extras_module(tp))
+        try:
+            return dispatch_method(obj, *args, **kwargs)
+        except NotImplementedError:
+            msg = f"No implementation for '{method.__name__}' extra for {cls.__name__} types"
+            for extra in extras:
+                if not extra.imported:
+                    try:
+                        if check_package_exists_on_pypi(extra.pypi):
+                            msg += (
+                                f'. An "extras" package exists on PyPI ({extra.pypi}), '
+                                "which may contain an implementation, try installing it "
+                                f"(e.g. 'pip install {extra.pypi}') and check again"
+                            )
+                    except urllib.error.URLError:
+                        msg += (
+                            '. Was not able to check whether an "extras" package '
+                            f"({extra.pypi}) exists on PyPI or not"
+                        )
+            raise FileFormatsExtrasError(msg)
+
+    decorated.register = dispatch_method.register
+    return decorated
